// Copyright (c) 2012, the Dart project authors.  Please see the AUTHORS file
// for details. All rights reserved. Use of this source code is governed by a
// BSD-style license that can be found in the LICENSE file.

library sunflower;

import 'dart:html' hide Range;
import 'dart:math';
import 'package:fields/fields.dart';

const int SCALE_FACTOR = 4;
const num TAU = PI * 2;
const int MAX_D = 300;
const num centerX = MAX_D / 2;
const num centerY = centerX;

class Sunflower {
  @FieldEditorType(ColorEditor)
  String color = "#ff8000";
  @FieldEditorType(RangeEditor) 
  @Range(min: 0, max: 1000)
  int seeds = 500;
  @FieldEditorType(NumberEditor)
  @Range(min: 1, max:3)
  int seedRadius = 2;
}

final InputElement slider = querySelector("#slider");
final Element notes = querySelector("#notes");
final ObjectEditor editor = new ObjectEditor(const ObjectEditorDarkResources());

final num PHI = (sqrt(5) + 1) / 2;
final Sunflower sunflower = new Sunflower();

final CanvasRenderingContext2D context =
  (querySelector("#canvas") as CanvasElement).context2D;

void main() {
  registerFieldsCustomElements();
  editor.selected = sunflower;
  querySelector("#editor").append(editor);
  
  editor.selectedChanged.listen((_) {
    slider.valueAsNumber = sunflower.seeds;
    draw();
  });
  
  slider.onChange.listen((e) {
    sunflower.seeds = int.parse(slider.value);
    editor.refresh();
    draw();
  });
  
  slider.valueAsNumber = sunflower.seeds;
  draw();
}

/// Draw the complete figure for the current number of seeds.
void draw() {
  context.clearRect(0, 0, MAX_D, MAX_D);
  for (var i = 0; i < sunflower.seeds; i++) {
    final num theta = i * TAU / PHI;
    final num r = sqrt(i) * SCALE_FACTOR;
    drawSeed(centerX + r * cos(theta), centerY - r * sin(theta));
  }
  notes.text = "${sunflower.seeds} seeds";
}

/// Draw a small circle representing a seed centered at (x,y).
void drawSeed(num x, num y) {
  context..beginPath()
         ..lineWidth = 2
         ..fillStyle = sunflower.color
         ..strokeStyle = sunflower.color
         ..arc(x, y, sunflower.seedRadius, 0, TAU, false)
         ..fill()
         ..closePath()
         ..stroke();
}
